<?php

namespace Drupal\views_cache_bully\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Views Cache Bully settings for this site.
 */
class ViewsCacheBullySettingsForm extends ConfigFormBase {

  /**
   * The Config Factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a ViewsCacheBullySettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ConfigFactoryInterface $config_factory, DateFormatterInterface $date_formatter, ModuleHandlerInterface $module_handler) {
    $this->configFactory = $config_factory;
    $this->dateFormatter = $date_formatter;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('date.formatter'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['views_cache_bully.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'views_cache_bully_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('views_cache_bully.settings');
    $options = [];
    foreach ([60, 300, 600, 1800, 3600, 21600, 43200, 86400] as $interval) {
      $options[$interval] = $this->dateFormatter->formatInterval($interval, 1);
    }
    $form['views_cache_bully_results_lifespan'] = [
      '#type' => 'select',
      '#title' => $this->t('Query results'),
      '#description' => $this->t('The length of time raw query results should be cached.'),
      '#options' => $options,
      '#default_value' => $config->get('results_lifespan'),
    ];
    $form['views_cache_bully_output_lifespan'] = [
      '#type' => 'select',
      '#title' => $this->t('Rendered output'),
      '#description' => $this->t('The length of time rendered HTML output should be cached.'),
      '#options' => $options,
      '#default_value' => $config->get('output_lifespan'),
    ];
    $form['views_cache_bully_exempt_exposed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Exempt all views with exposed forms from bullying'),
      '#description' => $this->t('Views using exposed forms will not be cached by Views Cache Bully.'),
      '#default_value' => $config->get('exempt_exposed'),
    ];
    foreach (Views::getAllViews() as $view) {
      // Only get views that must have fields.
      if (!empty($view->getDisplay('default')['display_options']['fields'])) {
        $views[$view->id()] = $view->label();
      }
    }
    $form['views_cache_bully_exemptions'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Exempt the following views from bullying'),
      '#description' => $this->t('Checked views will not be cached.'),
      '#options' => $views,
      '#default_value' => $config->get('exemptions') ? $config->get('exemptions') : [],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $values = $form_state->getValues();
    $config = $this->config('views_cache_bully.settings');
    $config->set('results_lifespan', $values['views_cache_bully_results_lifespan'])
      ->set('output_lifespan', $values['views_cache_bully_output_lifespan'])
      ->set('exempt_exposed', $values['views_cache_bully_exempt_exposed'])
      ->set('exemptions', $values['views_cache_bully_exemptions'])
      ->save();
  }

}
